<?php
/**
	Plugin Name: Three Column Screen Layout
	Plugin URI: http://wordpress.org/plugins/three-column-screen-layout/
	Description: Three, four and five column screen layouts for the post editor.
	Version: 3.0
	Author: Chad Hovell
	Author URI: http://www.chadhovell.com.au
	License: GPLv2 or later


	Copyright 2014 Chad Hovell  (email: chadhovell@gmail.com)

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License, version 2, as 
	published by the Free Software Foundation.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

if (!class_exists('Three_Column_Screen_Layout')) {
	class Three_Column_Screen_Layout {
		public function __construct() {
			register_activation_hook(__FILE__, array($this, 'activate'));
			register_deactivation_hook(__FILE__, array($this, 'deactivate'));
			if (function_exists('get_bloginfo') && version_compare(get_bloginfo('version'), '3.4') >= 0) {
				global $pagenow;
				if (is_admin() && in_array($pagenow, array('post.php', 'post-new.php'))) {
					add_action('admin_head', array($this, 'admin_head'));
					add_action('admin_footer', array($this, 'admin_footer'));
					add_action('admin_enqueue_scripts', array($this, 'admin_scripts'));
				}
			}
		}
		
		public function activate() {
			global $wpdb;
			//update metabox placement
			$wpdb->query("UPDATE wp_usermeta SET meta_value = replace(meta_value, 's:8:\"advanced\"', 's:5:\"side3\"') WHERE meta_key LIKE 'meta-box-order_%'");
		}
		
		public function deactivate() {
			global $wpdb;
			//reset metabox positions
			$wpdb->query("UPDATE wp_usermeta SET meta_value = replace(meta_value, 's:5:\"side3\"', 's:6:\"normal\"') WHERE meta_key LIKE 'meta-box-order_%'");
			$wpdb->query("UPDATE wp_usermeta SET meta_value = replace(meta_value, 's:5:\"side4\"', 's:6:\"normal\"') WHERE meta_key LIKE 'meta-box-order_%'");
		}
		
		public function admin_head() {
			ob_start();
			add_screen_option('layout_columns', array('max' => 24, 'default' => 2));
		}
		
		public function admin_footer() {
			$content = ob_get_contents();
			ob_end_clean();
			echo $this->splice_columns($content);
		}
		
		public function admin_scripts() {
			wp_enqueue_style('Three_Column_Screen_Layout-style', plugins_url('/style2.css', __FILE__));
		}
		
		public function splice_columns($content) {
			//find html locations
			$len = 				strlen($content);
			$pref_start = 		strpos($content, '<h5 class="screen-layout"');
			$pref_end = 		strpos($content, 'screenoptionnonce', $pref_start);
			$post_body_start = 	strpos($content, '<div id="post-body"', $pref_start);
			$col_start = 		strpos($content, 'columns-', $post_body_start) + 8;
			$cont2_start = 		strpos($content, '<div id="postbox-container-2"', $post_body_start);
			
			if ($pref_start && $pref_end && $cont2_start) {
				//find preferences
				$pref = substr($content, $pref_start, $pref_end - $pref_start);
				$oldpref = $pref;
				//find selected column
				$cols = preg_match("/value='(\d+)'[\r\n\s]+checked/", $pref, $matches) ? $matches[1] : 2;
				//splice in the icons
				$pref = preg_replace('/\/>[\r\n\s]+\d+[\s]+<\/label>/', '/><span class="columns-prefs-icon"></span></label>', $pref);
				$pref = preg_replace("/columns-prefs'>[^<]+<label/", "columns-prefs'><label", $pref);
				
				if ($pref != $oldpref) {
					global $post_type;
					global $post;
					
					//build new postboxes
					ob_start();
					do_action('do_meta_boxes', $post_type, 'side3', $post);
					do_meta_boxes($post_type, 'side3', $post);
					$side3 = '<div id="postbox-container-3" class="postbox-container">'.ob_get_contents().'</div>';
					ob_end_clean();
					
					ob_start();
					do_action('do_meta_boxes', $post_type, 'side4', $post);
					do_meta_boxes($post_type, 'side4', $post);
					$side4 = '<div id="postbox-container-4" class="postbox-container">'.ob_get_contents().'</div>';
					ob_end_clean();
					
					//splice content
					$content = substr_replace($content, $side3.$side4, $cont2_start, 0);
					$content = substr_replace($content, $cols, $col_start, 1);
					$content = substr_replace($content, $pref, $pref_start, $pref_end - $pref_start);
				}
			}
			return $content;
		}
	}
	
	$Three_Column_Screen_Layout = new Three_Column_Screen_Layout();
}
?>